// Test WASM module with invalid import name starting with 'wasm:'
import '../common/index.mjs';
import { spawnSyncAndAssert } from '../common/child_process.js';
import * as fixtures from '../common/fixtures.mjs';

spawnSyncAndAssert(
  process.execPath,
  ['--no-warnings', fixtures.path('es-modules/test-wasm-reject-wasm-import-names.mjs')],
  { status: 1, stderr: /Invalid Wasm import name/ }
);
