// META: title=WebCryptoAPI: verify() Using EdDSA with small-order points
// META: script=../util/helpers.js
// META: script=eddsa_vectors.js
// META: script=eddsa_small_order_points.js
// META: timeout=long

run_test();
